/*
 * $Id: main.c,v 1.1 2003/11/14 10:03:37 alexanto Exp $
 *
 * $Doctor Gauge (C) 2000-2003 A. Antonini, All rights reserved $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Desc: Main dialog implementation $
 *
 * $Log: main.c,v $
 * Revision 1.1  2003/11/14 10:03:37  alexanto
 * Initial revision
 *
 */
#include <windows.h>
#include <commctrl.h>
#include <htmlhelp.h>
#include "utils.h"
#include "regscan.h"
#include "resource.h"
#include "globals.h"
#include "about.h"
#include "report.h"
#include "avl.h"
#include "shfile.h"

/*
 * structure to hold gauge info
 */
typedef struct
{
	char szGauge[_MAX_FNAME];
	int use;
} gauge_t;

/*
void TestDelete(HWND hWnd)
{
	char gauge_path[1024];
	char subpath[1024],path[1024];
	BOOL b;
	HANDLE fh;
	WIN32_FIND_DATA fd;
	shell_op_t ptr;
	
	lstrcpy(gauge_path,szFsPath);
	VerifySlash(gauge_path);
	lstrcat(gauge_path,szGauges);

	lstrcpy(path,gauge_path);
	
	lstrcat(path,"*.gau");

	ShellFileOpInit(hWnd, &ptr);
	fh = FindFirstFile(path,&fd);	// collect gauges
	b = INVALID_HANDLE_VALUE != fh;
	while (b)
	{
		lstrcpy(subpath,gauge_path);
		lstrcat(subpath,fd.cFileName);
		ShellFileOpAddFile(SH_SRC_FILE, subpath,&ptr);

		b = FindNextFile(fh,&fd);		// search next gauge
	}
	FindClose(fh);
	ShellFileOpDeleteFiles(&ptr);
	ShellFileOpDestroy(&ptr);


}
*/

/*
 * global variables
 */
static AVL_TREE *ExcludeFile;
static AVL_TREE *Gauges;	// of gauge_t

/*
 * compare function for exclude file avl_tree
 */
static int CmpFunc(void *p1, void *p2)
{
	return lstrcmp((char *)p1, (char *)p2);
}





/*
 * compare function for exclude file avl_tree
 */
static int GaugeCmpFunc(void *p1, void *p2)
{
	gauge_t *ptr1 = (gauge_t *)p1;
	gauge_t *ptr2 = (gauge_t *)p2;
	return lstrcmp(ptr1->szGauge, ptr2->szGauge);
}

/*
 * add gauge list to list view
 */
static void AddGaugeToListView(void *param,void *node)
{
	HWND hListView = (HWND)param;
	gauge_t *ptr = (gauge_t *)node;
	LVITEM lv;
	char buff[32];
	int i;
	int nCount = SendMessage(hListView,LVM_GETITEMCOUNT,0,0);
	if (!avl_findsym(ExcludeFile,ptr->szGauge))		// if gauge not in exclude list
	{												// then add it to list view
		
		lv.mask = LVIF_TEXT;
		lv.iItem = nCount; 
		lv.iSubItem = 0; 
		lv.state = 0; 
		lv.stateMask = 0; 
		lv.pszText = ptr->szGauge; 
		lv.cchTextMax = lstrlen(ptr->szGauge); 
		lv.iImage = 0; 
		lv.lParam = 0;								// set first column
		SendMessage(hListView, LVM_INSERTITEM, 0, (LPARAM)&lv);

		lv.mask = LVIF_TEXT;
		lv.iItem = nCount; 
		lv.iSubItem = 1; 
		lv.state = 0; 
		lv.stateMask = 0; 
		i = wsprintf(buff,"%d",ptr->use);
		lv.pszText = buff; 
		lv.cchTextMax = i; 
		lv.iImage = 0; 
		lv.lParam = 0;								// set second column
		SendMessage(hListView, LVM_SETITEM, 0, (LPARAM)&lv);
	}
}

/*
 * set caption bar title depending on which simulator we have
 */
static void SetCaption(HWND hWnd, char *lpCaption, int nCode)
{
	if (nCode==RESULT_FS2K2)
		lstrcat(lpCaption, " - FS2002");
	else if (nCode==RESULT_FS2K4)
		lstrcat(lpCaption, " - FS2004");
	SetWindowText(hWnd,lpCaption);					// set caption text
}

/*
 * examinate gauges withinh a window section of a panel file. 
 * if gauges are found compare them to our gauge list
 */
static void ExaminatePanelGauges(char *section, char *panelfile)
{
	int i;
	char *p;
	char *token1, *token2;
	gauge_t *gauge;
	char gauge_name[80];
	char *ptr = (char *)GlobalAllocPtr(BufSize);
	if (GetPrivateProfileString(section, NULL, NULL, ptr, BufSize,
        panelfile) != 0)
	{
		p=ptr;
		while (*p)									// for each section entry
		{
			CharLower(p);
			if (strstr(p,"gauge"))					// if we have got a gauge
			{
				i=GetPrivateProfileString(section,p,"",gauge_name,
					sizeof(gauge_name),panelfile);	// get the gauge name
				if (i)								// if valid gauge
				{
					CharLower(gauge_name);			// ensure lower case
					token1 = strtok(gauge_name,",");// extract subtoken
					if (token1)						// if valid token
					{
						token2 = strtok(token1,"!");// extract other subtoken
						if (token2)
						{
							gauge = (gauge_t *)avl_findsym(Gauges,token2);
							if (gauge)
								gauge->use++;		// increment use
						}
						else
						{
							gauge = (gauge_t *)avl_findsym(Gauges,token1);
							if (gauge)
								gauge->use++;		// increment use
						}

					}
					else
					{
						gauge = (gauge_t *)avl_findsym(Gauges,gauge_name);
						if (gauge)
							gauge->use++;			// increment use

					}
				}

			}
			p+=lstrlen(p)+1;
		}
	}
	GlobalFreePtr(ptr);
}


/*
 * examinate panel cfg file - if window titles section is found,
 * them process it
 */
static void ExaminatePanelCfg(char *panelfile)
{
	char *p;
	char *ptr = (char *)GlobalAllocPtr(BufSize);
	if (GetPrivateProfileString("Window Titles", NULL, 
		NULL, ptr, BufSize, panelfile) != 0)		// we found windows sections
	{
		p = ptr;
		while (*p)									// for each window section
		{
			ExaminatePanelGauges(p,panelfile);		// parse gauges within this
													// window
			p+=lstrlen(p)+1;						// increment to next token
		}

	}
	GlobalFreePtr(ptr);
}


/*
 * recursively scan hard disk, looking for panel.cfg files
 */
static void ScanPanelCfgFiles(HWND hWnd, HWND hProgress, char *startpath)
{
	HANDLE fh;
	WIN32_FIND_DATA fd;
	char path[2048];
	char subpath[2048];
	char progress_buff[2048];
	BOOL b;

	// recurse directories first
    lstrcpy(path, startpath);
    VerifySlash(path);
	lstrcat(path,"*.*");

	fh = FindFirstFile(path,&fd);
	b=fh!=INVALID_HANDLE_VALUE;

	while (b)										// do sub-directory recursion
	{
		if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (lstrcmp(fd.cFileName,".")!=0 &&
	            lstrcmp(fd.cFileName,"..")!=0)		// is a sub directory
			{
		    	lstrcpy(subpath, startpath );
                VerifySlash(subpath);
                lstrcat(subpath, fd.cFileName );
				lstrcpy(progress_buff,"Scanning ");
				lstrcat(progress_buff,subpath);
				SetWindowText(hProgress,subpath);
				ScanPanelCfgFiles(hWnd, hProgress, subpath);
			}
		}
	    b = FindNextFile(fh,&fd);
	}
	FindClose(fh);


    lstrcpy(path, startpath);						// find panel cfg files now
    VerifySlash(path);
	lstrcat(path,"panel.cfg");
	fh = FindFirstFile(path,&fd);

	b=fh!=INVALID_HANDLE_VALUE;
	while (b)
	{
		lstrcpy(subpath,startpath);					// produce path to panel.cfg
	    VerifySlash(subpath);
		lstrcat(subpath,fd.cFileName);				// if file is valid
		ExaminatePanelCfg(subpath);					// then examinate it
	    b = FindNextFile(fh,&fd);
	}
	FindClose(fh);

}

/*
 * make sure gau extension is stripped
 */
static char *StripGauExt(char *str)
{
	char *ptr = strrchr(str,'.');
	if (ptr)
	{
		if (lstrcmp(ptr,".gau")==0)
			*ptr = '\0';

	}
	return str;
}

/*
 * parse exclusion file out of exclude.txt
 */
void ParseExcludeFile(void)
{
	HANDLE fh, fm;
	WIN32_FIND_DATA fd;
	char *buff;
	char *start;
	char *end;
	char *P;
	char *str;
	char exclude_path[_MAX_PATH];
	DWORD dwSize, dwErr;

	GetAppPath(exclude_path);
	VerifySlash(exclude_path);
	lstrcat(exclude_path,"exclude.lst");

	fh=FindFirstFile(exclude_path,&fd);				// used to get file size
	if (fh==INVALID_HANDLE_VALUE)					// file not found
	{
		return;
	}
	dwSize=fd.nFileSizeLow;							// get size of file to map

	FindClose(fh);
	
	
	fh = CreateFile(exclude_path,					// open the file
		GENERIC_READ, FILE_SHARE_READ, 
		NULL, OPEN_EXISTING, 
		FILE_ATTRIBUTE_READONLY, NULL);
	if (fh == INVALID_HANDLE_VALUE)					// if error
		return;

	fm = CreateFileMapping(fh, NULL,				// create the file mapping
		PAGE_READONLY, 0, 0, "exclude_file");
	dwErr = GetLastError();
	if ((fm != NULL) && (dwErr == ERROR_ALREADY_EXISTS)) 
	{
		CloseHandle(fm);							// if mapping already exists
		CloseHandle(fh);
		return;
	} 
	else if (fm == NULL)
	{
		CloseHandle(fh);							// if error
		return;
	}	

	buff = (char *)MapViewOfFile(fm,				// map file to memory
		FILE_MAP_READ, 0, 0, 0); 
	if (buff == NULL)								// if error
	{
		CloseHandle(fm);
		CloseHandle(fh);							// return failure
		return;
	}
	
	start = buff;									// mark start of file
	end = buff + dwSize;							// mark end of file

	
	while (start<end)								// do parsing
	{
		if (*start==';')							// we found a comment
		{
			while (*start!='\r' && *start!='\n')
				start++;
		}
		else if (*start=='\r')						// cr encountered
			start++;
		else if (*start=='\n')						// lf encountered
			start++;
		else
		{
			P = start;
			while (*start!='\r' && *start!='\n' 
				&& *start!=';')						// while the string is valid
				start++;							// increment it
			
			
			str = (char *)avl_newnode((start-P)+1);
			memset(str,0x0,(start-P)+1);
			MoveMemory(str,P,start-P);
			strrtrim(str);
			CharLower(str);
			StripGauExt(str);						// compose the gauge string
			avl_insert(ExcludeFile,str);			// add it to search tree
		}
	}

	UnmapViewOfFile(buff);							// do memory cleanup
	CloseHandle(fm);				
	CloseHandle(fh);
}

/*
 * scan for gauge files
 */
static void DoScanGauges(HWND hList, HWND hProgress)
{
	char startpath[2048];
	HANDLE fh;
	WIN32_FIND_DATA fd;
	BOOL b;
	gauge_t *ptr;
	char aircraft_path[2048];
	
	SendMessage(hList,LVM_DELETEALLITEMS,0,0);		// clean list-view
	
	avl_empty(ExcludeFile,avl_freenode);			// clean existing exclude file
	avl_empty(Gauges,avl_freenode);					// clean gauges
	
	lstrcpy(startpath,szFsPath);					// form gauge path
	VerifySlash(startpath);
	lstrcat(startpath,szGauges);
	lstrcat(startpath,"*.gau");
	
	fh = FindFirstFile(startpath,&fd);				// collect gauges
	b = INVALID_HANDLE_VALUE != fh;
	while (b)
	{
		if (NULL!=(ptr = (gauge_t *)avl_newnode(sizeof(gauge_t))))
		{
			lstrcpy(ptr->szGauge,fd.cFileName);
			CharLower(ptr->szGauge);
			StripGauExt(ptr->szGauge);
			ptr->use=0;
			avl_insert(Gauges,ptr);					// add this gauge to list
		}
		b = FindNextFile(fh,&fd);					// search next gauge
	}
	FindClose(fh);
	
	ParseExcludeFile();								// check exclude.lst

	lstrcpy(aircraft_path,szFsPath);				// compose path to aircrafts
	VerifySlash(aircraft_path);
	lstrcat(aircraft_path,"aircraft");
	ScanPanelCfgFiles(hList, hProgress, 
		aircraft_path);								// and do the scanning
	avl_traverse(Gauges,INORDER,AddGaugeToListView,(void *)hList);
	
	SetWindowText(hProgress,"Ready");				// done

}

/*
 * kill the gauge files
 */
void DoKillGaugeFiles(HWND hWnd, HWND hProgress, int method)
{
	LVITEM item;
	gauge_t *ptr;
	char buff[_MAX_FNAME];							// hold lv item text
	char gauge_home[1024];
	char moved_path[1024];
	char dest_path[1024];
	char src_path[1024];
	char wtext[128];
	int i, gf=0;
	int nCount = ListView_GetItemCount(hWnd);		// get count of lv items
	shell_op_t shell;
	lstrcpy(gauge_home,szFsPath);
	VerifySlash(gauge_home);
	lstrcat(gauge_home,szGauges);					// get gauges dource dir
		

	switch (method)
	{
	case 0:											// erase the gauges
		ShellFileOpInit(hWnd, &shell);				// initialize shell operation
		for (i=0; i<nCount; i++)					// for each lv item
		{

			item.iItem = i;	
			item.iSubItem = 0;
			item.pszText = buff;
			item.mask = LVIF_TEXT;
			item.cchTextMax = sizeof(buff);
			ListView_GetItem(hWnd,&item);			// get current gauge name
			if (ptr=avl_findsym(Gauges,item.pszText))// if entry found
			{
				if (ptr->use==0)					// if unused gauge
				{
					gf=1;							// set found flag to true
					lstrcpy(src_path,gauge_home);	// form source path
					lstrcat(src_path,item.pszText);
					lstrcat(src_path,".gau");
					wsprintf(wtext,"Erasing %s",item.pszText);
					ShellFileOpAddFile(SH_SRC_FILE, 
						src_path,&shell);			// add it to trash list

				}
			}
		}
		if (gf)
			ShellFileOpDeleteFiles(&shell);			// send to recycle bin
		ShellFileOpDestroy(&shell);					// free shell operation
		break;
	
	case 1:											// move the gauges
		GetAppPath(moved_path);
		VerifySlash(moved_path);
		lstrcat(moved_path,"moved");
		ForceDirectories(moved_path);				// ensure dest dir exists
		
		for (i=0; i<nCount; i++)					// for each lv item
		{
			item.iItem = i;	
			item.iSubItem = 0;
			item.pszText = buff;
			item.mask = LVIF_TEXT;
			item.cchTextMax = sizeof(buff);
			ListView_GetItem(hWnd,&item);			// get current gauge name
			if (ptr=avl_findsym(Gauges,item.pszText))// if entry found
			{
				if (ptr->use==0)					// if unused gauge
				{
					lstrcpy(src_path,gauge_home);	// form source path
					lstrcat(src_path,item.pszText);
					lstrcat(src_path,".gau");

					lstrcpy(dest_path,moved_path);	// form destination path
					VerifySlash(moved_path);
					lstrcat(dest_path,item.pszText);
					lstrcat(dest_path,".gau");
					wsprintf(wtext,"Moving %s",item.pszText);
					SetWindowText(hProgress,wtext);
					MoveFile(src_path,dest_path);	// move file
				}
			}
		}
		break;
	}
}


/*
 * window procedure routine
 */
int WINAPI MainDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	ver_info_t vi;
	HMENU hMenu;
	HWND hListView, hProgress;
	int i;
	LVCOLUMN col;
	TCHAR buff[32];
	HELPINFO *lpHelpInfo;
	switch (uMsg)
	{
	case WM_INITDIALOG:
		ExcludeFile = avl_init(CmpFunc);
		Gauges = avl_init(GaugeCmpFunc);
		hProgress = GetDlgItem(hWnd, IDC_PROGRESS);

		SetClassLong(hWnd,GCL_HICON,				// add my icon to system menu
			(int)LoadIcon(GlobalInstance,
			MAKEINTRESOURCE(IDI_MAINICON)));
		CenterWindow(hWnd);							// center window on the screen
		GetFileVersion(&vi);						// get exefile information
		SetCaption(hWnd,vi.szProductName,lParam);
		hMenu = GetSystemMenu(hWnd,FALSE);			// append 'about' to system menu
		if (hMenu)
		{
			GetWindowText(GetDlgItem(hWnd,IDC_ABOUT),buff,sizeof(buff));
			AppendMenu(hMenu,MF_SEPARATOR,0,"");
			AppendMenu(hMenu,MF_STRING,IDC_ABOUT,buff);

		}

		
		hListView=GetDlgItem(hWnd,IDC_GAUGELIST);	// set up list view columns
		col.mask = LVCF_TEXT | LVCF_WIDTH;
		col.fmt = LVCFMT_LEFT;
		col.cx = GetIniInt(szSettings,szPanelColWidth,200);
		col.pszText = "Panels";
		col.cchTextMax = 6;
		col.iSubItem = 0;
		col.iImage = 0;
	    col.iOrder = 0;
		
		SendMessage(hListView,LVM_INSERTCOLUMN,0,(LPARAM)&col);

		col.mask = LVCF_TEXT | LVCF_WIDTH;
		col.fmt = LVCFMT_LEFT;
		col.cx = GetIniInt(szSettings,szUseColWidth,100);
		col.pszText = "Use";
		col.cchTextMax = 3;
		col.iSubItem = 0;
		col.iImage = 0;
	    col.iOrder = 0;
		
		SendMessage(hListView,LVM_INSERTCOLUMN,1,(LPARAM)&col);

		
		DoScanGauges(hListView,hProgress);			// scan gauge folder
		
		i = GetIniInt(szSettings,szMoveMethod,1);	// see which erase method was adopted
		if (i<0 || i>1)
			i=1;
		if (i==0)									// erase method selected
			SendMessage(GetDlgItem(hWnd,IDC_RADIOKILL),
			BM_SETCHECK,BST_CHECKED,0);
		else										// move method selected
			SendMessage(GetDlgItem(hWnd,IDC_RADIO_MOVE),
			BM_SETCHECK,BST_CHECKED,0);
		return FALSE;

	case WM_HELP:
		lpHelpInfo = (HELPINFO *)lParam;
		switch (lpHelpInfo->iCtrlId)
		{
		case IDC_RADIOKILL:
		case IDC_RADIO_MOVE:
			lpHelpInfo->iCtrlId=IDC_RADIOKILL;
			break;
		case IDHELP:
		case IDCANCEL:
			lpHelpInfo->iCtrlId=IDD_MAIN_DIALOG;
			break;
		}
		HtmlHelp(hWnd, szHelpFile, HH_HELP_CONTEXT, lpHelpInfo->iCtrlId); 
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDHELP:
			HtmlHelp(hWnd, szHelpFile, HH_HELP_CONTEXT, IDD_MAIN_DIALOG); 
			return TRUE;
		case IDC_SCAN:
			i = TestFlightSimulator(hWnd);
			if (i!=-1)
			{
				hListView=GetDlgItem(hWnd,IDC_GAUGELIST);
				hProgress = GetDlgItem(hWnd, IDC_PROGRESS);
				GetFileVersion(&vi);				// get exefile information
				SetCaption(hWnd,vi.szProductName,i);// update caption bar
				DoScanGauges(hListView,hProgress);	// re-scan gauge folder
			}
			return TRUE;
		case IDC_KILL:
			i = SendMessage(GetDlgItem(hWnd,IDC_RADIOKILL),BM_GETCHECK,0,0) ==
				BST_CHECKED ? 0 : 1;
			hListView=GetDlgItem(hWnd,IDC_GAUGELIST);
			hProgress = GetDlgItem(hWnd, IDC_PROGRESS);
			DoKillGaugeFiles(hListView,hProgress,i);
			DoScanGauges(hListView,hProgress);

			return TRUE;

		case IDC_ABOUT:								// display about box
			RunAboutBox(hWnd);
			return TRUE;
		case IDC_REPORT:							// display report dialog
			RunReportDialog(hWnd);
			return TRUE;
		case IDCANCEL:								// close application
			hListView=GetDlgItem(hWnd,IDC_GAUGELIST);// save settings

			i = SendMessage(GetDlgItem(hWnd,IDC_RADIOKILL),BM_GETCHECK,0,0) ==
				BST_CHECKED ? 0 : 1;
			SetIniInt(szSettings,szMoveMethod,i);
			SetIniInt(szSettings,szPanelColWidth,
				SendMessage(hListView,LVM_GETCOLUMNWIDTH,0,0));
			SetIniInt(szSettings,szUseColWidth,
				SendMessage(hListView,LVM_GETCOLUMNWIDTH,1,0));

			avl_empty(ExcludeFile,avl_freenode);	// free memory from exclude file
			GlobalFreePtr(ExcludeFile);				// free exclude file itself

			avl_empty(Gauges,avl_freenode);			// free memory from exclude file
			GlobalFreePtr(Gauges);					// free exclude file itself

			EndDialog(hWnd,IDCANCEL);
			return TRUE;
		}
		break;

	case WM_SYSCOMMAND:
		switch (wParam)
		{
		case IDC_ABOUT:								// exec about (from system menu)
			RunAboutBox(hWnd);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

/*
 * run main dialog (main application loop)
 */
int RunMainDialog(int nParam)
{
	return DialogBoxParam(GlobalInstance,
		MAKEINTRESOURCE(IDD_MAIN_DIALOG),NULL,
		MainDialogProc, nParam);
}
