/*
 * $Id: report.c,v 1.1 2003/11/14 10:03:37 alexanto Exp $
 *
 * $Doctor Gauge (C) 2000-2003 A. Antonini, All rights reserved $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Desc: Report dialog implementation $
 *
 * $Log: report.c,v $
 * Revision 1.1  2003/11/14 10:03:37  alexanto
 * Initial revision
 *
 */
#include <windows.h>
#include <commctrl.h>
#include <stdlib.h>
#include "utils.h"
#include "resource.h"
#include "report.h"
#include "globals.h"

HIMAGELIST hIml;

static int PanelCount;
static int CraftCount;

static char *PanelCfgFromAircraftCfg(const char *instr, char *outstr, char *panel_ext)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char name[_MAX_FNAME];
	char ext[_MAX_EXT];
	
	_splitpath(instr,drive,dir,name,ext);
	lstrcpy(outstr,drive);
	lstrcat(outstr,dir);
	lstrcat(outstr,"PANEL");
	if (*panel_ext)
	{
		lstrcat(outstr,".");
		lstrcat(outstr,panel_ext);
	}
	lstrcat(outstr,"\\panel.cfg");
	return outstr;
}

/*
 *	examinate gauges within a single window entry
 */
static void ExaminateGauges(char *inifile, char *section, HWND hTree, HTREEITEM hRoot)
{
	TVINSERTSTRUCT struc;
	HTREEITEM hChild;
	char *p;
	char *str;
	char gauge_name[80];
	int i;
	char *ptr = (char *)GlobalAllocPtr(BufSize);
	if (GetPrivateProfileString(section, NULL, NULL, ptr, BufSize,
        inifile) != 0)
	{
		p=ptr;
		while (*p)
		{
			CharLower(p);
			if (strstr(p,"gauge"))		// if we have got a gauge
			{
				i=GetPrivateProfileString(section,p,"",gauge_name,
					sizeof(gauge_name),inifile);
				if (i)					// if valid gauge
				{
					struc.hParent = hRoot;
					struc.hInsertAfter = NULL;
					struc.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
					struc.item.hItem=NULL;
					struc.item.state=0;
					struc.item.stateMask=0;
					str = strtok(gauge_name,",");
					if (str!=NULL)
					{
						struc.item.pszText=str;
						struc.item.cchTextMax=strlen(str);
					}
					else
					{
						struc.item.pszText=gauge_name;
						struc.item.cchTextMax=i;
					}
					struc.item.iImage=2;
					struc.item.iSelectedImage=2;
					struc.item.cChildren=0;
					struc.item.lParam=0;
					hChild=TreeView_InsertItem(hTree, &struc);
				}
			}
			p+=lstrlen(p)+1;
		}
	}
	GlobalFreePtr(ptr);
}

/*
 *	examinate entries within a single panel.cfg file
 */
static void ExaminatePanel(char *inifile, HWND hTree, HTREEITEM hRoot)
{
	int isWindow = TRUE;
	char *p;
	int i;
	char wintitle[80];
	TVINSERTSTRUCT struc;
	HTREEITEM hChild;
	char *ptr = (char *)GlobalAllocPtr(BufSize);
	if (GetPrivateProfileString("Window Titles", NULL, NULL, ptr, BufSize,
        inifile) != 0)
	{
		p = ptr;
		while (*p)
		{
			i=GetPrivateProfileString("Window Titles",p,"",wintitle,sizeof(wintitle),
				inifile);
			if (i!=0)
			{
				struc.hParent = hRoot;
				struc.hInsertAfter = NULL;
				struc.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
				struc.item.hItem=NULL;
				struc.item.state=0;
				struc.item.stateMask=0;
				struc.item.pszText=wintitle;
				struc.item.cchTextMax=i;
				struc.item.iImage=3;
				struc.item.iSelectedImage=3;
				struc.item.cChildren=0;
				struc.item.lParam=0;
				hChild=TreeView_InsertItem(hTree, &struc);
				ExaminateGauges(inifile,p,hTree,hChild);
			}
			p+=lstrlen(p)+1;
			PanelCount++;
		}

	}
	GlobalFreePtr(ptr);
}


/*
 * add all panel gauges inside aircraft.cfg to tree view
 */
static void EnumeratePanels(char *sections, char *inifile, HWND hTree, HTREEITEM hParent)
{
	TVINSERTSTRUCT struc;
	HTREEITEM hRoot;
	char *ptr = sections;
	char title[80];
	char panelpath[512];
	char panelext[16];
	while (*ptr)
	{
		CharLower(ptr);
		if (strstr(ptr,"fltsim."))	// we found an entry
		{
			GetPrivateProfileString(ptr,"Title","",title,sizeof(title),inifile);
			if (*title)		// we found a valid aircraft
			{
				struc.hParent = hParent;
				struc.hInsertAfter = NULL;
				struc.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
				struc.item.hItem=NULL;
				struc.item.state=0;
				struc.item.stateMask=0;
				struc.item.pszText=title;
				struc.item.cchTextMax=lstrlen(title);
				struc.item.iImage=1;
				struc.item.iSelectedImage=0;
				struc.item.cChildren=0;
				struc.item.lParam=0;
				hRoot=TreeView_InsertItem(hTree, &struc);
				CraftCount++;
	
				// get path to panel cfg
				GetPrivateProfileString(ptr,"panel","",panelext,sizeof(panelext),
					inifile);
				PanelCfgFromAircraftCfg(inifile,panelpath,panelext);	// produce path to panel
				ExaminatePanel(panelpath,hTree,hRoot);
			}
		}
		ptr+=lstrlen(ptr)+1;
	}
}


/*
 * produce a tree view of all aircraft with relative gauges
 */
static void FillTreeView(HWND hTree, char *startpath, HTREEITEM hRoot)
{

	HANDLE fh;
	WIN32_FIND_DATA fd;
	char path[2048];
	char subpath[2048];
	BOOL b;
	char *ptr;

	// recurse directories first
    lstrcpy(path, startpath);
    VerifySlash(path);
	lstrcat(path,"*.*");

	fh = FindFirstFile(path,&fd);
	b=fh!=INVALID_HANDLE_VALUE;

	while (b)
	{
		if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (lstrcmp(fd.cFileName,".")!=0 &&
	            lstrcmp(fd.cFileName,"..")!=0)	// is a sub directory
			{
		    	lstrcpy(subpath, startpath );
                VerifySlash(subpath);
                lstrcat(subpath, fd.cFileName );
			    FillTreeView(hTree,subpath, hRoot);
			}
		}
	    b = FindNextFile(fh,&fd);
	}
	FindClose(fh);


	// find aircraft cfg file now
    lstrcpy(path, startpath);
    VerifySlash(path);
	lstrcat(path,"aircraft.cfg");
	fh = FindFirstFile(path,&fd);

	b=fh!=INVALID_HANDLE_VALUE;
	while (b)
	{
		lstrcpy(subpath,startpath);		// produce path to aircraft.cfg
	    VerifySlash(subpath);
		lstrcat(subpath,fd.cFileName);
		ptr = (char *)GlobalAllocPtr(BufSize);
		if (ptr)
		{
			GetPrivateProfileString(NULL, NULL, NULL, ptr,
				BufSize, subpath);
			EnumeratePanels(ptr,subpath,hTree, hRoot);
			GlobalFreePtr((void *)ptr);
		}

	    b = FindNextFile(fh,&fd);
	}
	FindClose(fh);
 
}

/*
 * get text width for list box horizontal extent
 */
static int GetLbWidth(HDC dc, char *str)
{
	SIZE size;
	size.cx=size.cy=0;
	GetTextExtentPoint32(dc,str,strlen(str),&size);
	return size.cx;
}

/*
 * fill list box with found gauges
 */
static void AddGaugesToListBox(HWND hWnd)
{
	char gauge_path[1024];
	HANDLE fh;
	WIN32_FIND_DATA fd;
	BOOL b;
	HFONT oldFont,font = (HFONT)SendMessage(hWnd,WM_GETFONT,0,0);
	int scrollw,w,cw = 0;
	HDC dc = GetDC(hWnd);	
	lstrcpy(gauge_path,szFsPath);
	VerifySlash(gauge_path);
	lstrcat(gauge_path,szGauges);
	lstrcat(gauge_path,"*.gau");
	oldFont=SelectObject(dc,font);
	fh = FindFirstFile(gauge_path,&fd);
	b = fh!=INVALID_HANDLE_VALUE;
	scrollw=GetSystemMetrics(SM_CYHSCROLL);
	while (b)
	{
		CharLower(fd.cFileName);
		SendMessage(hWnd,LB_ADDSTRING,0,(LPARAM)fd.cFileName);
		w=GetLbWidth(dc,fd.cFileName)+(scrollw-2);
		if (cw<w)
		{
			cw=w;
			SendMessage(hWnd,LB_SETHORIZONTALEXTENT,cw,0);
		}
		b=FindNextFile(fh,&fd);
	}
	FindClose(fh);
	SelectObject(dc,oldFont);
	ReleaseDC(hWnd,dc);
}


/*
 * create imagelist and add icons to it
 */
static HIMAGELIST CreateImageList(void)
{
	HICON hIcon;
	HIMAGELIST hImages = ImageList_Create(16,16, ILC_COLOR32 | ILC_MASK, 4 , 4);
	hIcon = LoadImage(GlobalInstance,MAKEINTRESOURCE(IDI_OPENFOLDER), IMAGE_ICON,
		16,16, LR_LOADTRANSPARENT);
	ImageList_AddIcon(hImages,hIcon);
	hIcon = LoadImage(GlobalInstance,MAKEINTRESOURCE(IDI_CLOSEFOLDER), IMAGE_ICON,
		16,16, LR_LOADTRANSPARENT);
	ImageList_AddIcon(hImages,hIcon);
	hIcon = LoadImage(GlobalInstance,MAKEINTRESOURCE(IDI_GAUGE), IMAGE_ICON,
		16,16, LR_LOADTRANSPARENT);
	ImageList_AddIcon(hImages,hIcon);
	hIcon = LoadImage(GlobalInstance,MAKEINTRESOURCE(IDI_WINDOW), IMAGE_ICON,
		16,16, LR_LOADTRANSPARENT);
	ImageList_AddIcon(hImages,hIcon);

	return hImages;

}


/*
 * window procedure routine
 */
int WINAPI ReportDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char acf_path[2048];
	HTREEITEM hRoot;
	TVINSERTSTRUCT struc;
	char buff[64];
	switch (uMsg)
	{
	case WM_INITDIALOG:
		/*hIml=ImageList_LoadImage(GlobalInstance,MAKEINTRESOURCE(IDB_IMAGELIST),
			16,4,RGB(255,0,255),IMAGE_BITMAP,LR_SHARED);*/
		hIml = CreateImageList();
		CenterWindow(hWnd);
		AddGaugesToListBox(GetDlgItem(hWnd,IDC_GAUGELISTBOX));
		
		lstrcpy(acf_path,szFsPath);		// build aircraft path
		VerifySlash(acf_path);
		lstrcat(acf_path,"aircraft");	// path now fs2kx\aircraft

		TreeView_SetImageList(GetDlgItem(hWnd,IDC_PANELS_TREE),hIml,
			TVSIL_NORMAL);

		// build tree list
		struc.hParent = NULL;
		struc.hInsertAfter = NULL;
		struc.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		struc.item.hItem=NULL;
		struc.item.state=0;
		struc.item.stateMask=0;
		struc.item.pszText="Planes";
		struc.item.cchTextMax=6;
		struc.item.iImage=1;
		struc.item.iSelectedImage=0;
		struc.item.cChildren=0;
		struc.item.lParam=0;
		CraftCount=PanelCount=0;
		hRoot=TreeView_InsertItem(GetDlgItem(hWnd,IDC_PANELS_TREE), &struc);

		FillTreeView(GetDlgItem(hWnd,IDC_PANELS_TREE), acf_path, hRoot);
		TreeView_Expand(GetDlgItem(hWnd,IDC_PANELS_TREE),hRoot,TVE_EXPAND);

		// set panels count
		wsprintf(buff,"%d panels, %d aircrafts",PanelCount,CraftCount);
		SetWindowText(GetDlgItem(hWnd,IDC_PANELS_COUNT),buff);
		
		wsprintf(buff,"%d gauges",
			SendMessage(GetDlgItem(hWnd,IDC_GAUGELISTBOX),LB_GETCOUNT,0,0));
		SetWindowText(GetDlgItem(hWnd,IDC_GAUGES_COUNT),buff);
		return FALSE;


	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			ImageList_Destroy(hIml);	// free memory from image list
			EndDialog(hWnd,IDOK);		// return OK
			return TRUE;
		case IDCANCEL:
			ImageList_Destroy(hIml);	// free memory from image list
			EndDialog(hWnd,IDCANCEL);	// return cancel

			return TRUE;
		}
		break;
	}

	return FALSE;
}

int RunReportDialog(HWND hWnd)
{
	return DialogBoxParam(GlobalInstance,MAKEINTRESOURCE(IDD_REPORT_DIALOG),
		hWnd,ReportDialogProc,0);

}