/*
 * $Id: shfile.c,v 1.2 2003/11/14 10:05:45 alexanto Exp Alessandro $
 *
 * $Doctor Gauge (C) 2000-2003 A. Antonini, All rights reserved $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Desc: A wrapper for SHFileOperation API $
 *
 * $Log: shfile.c,v $
 * Revision 1.2  2003/11/14 10:05:45  alexanto
 * +Id+Log
 *
 */
#include <windows.h>
#include <shellapi.h>
#include "utils.h"
#include "shfile.h"


//
// GrabMem( const int iSrcDest, const long lNum )
//
// function to grab some string space memory
//
static void GrabMem(const int iSrcDest, const long lNum, shell_op_t *ptr)
{
	char * pMem;
	long lOffset;

	//get current ptr
	if (iSrcDest == SH_SRC_FILE)
	{
		pMem = ptr->pFrom;
	} 
	else 
	{
		pMem = ptr->pTo;
	}

	if (pMem)	//some mem is already allocated!
	{
		//have to make sure our offset ptrs dont get screwed up...
		if (iSrcDest == SH_SRC_FILE) 
		{
			lOffset = (ptr->pSrc - pMem);
		} 
		else 
		{
			lOffset = (ptr->pDest - pMem);
		}

		//get more!
		pMem = (char *)GlobalReAllocPtr((void *)pMem, lNum);

		//reassign offset ptr, and max counts
		if (iSrcDest == SH_SRC_FILE) 
		{
			ptr->pSrc = pMem + lOffset;
			ptr->lMaxSrcCount = lNum;
		} 
		else 
		{
			ptr->pDest = pMem + lOffset;
			ptr->lMaxDestCount = lNum;
		}
	}
	else
	{
		//get first block
		pMem = (char *)GlobalAllocPtr(lNum);

		//assign offset ptr to start of block, and max counts
		if (iSrcDest == SH_SRC_FILE)
		{
			ptr->pSrc = pMem;
			ptr->lMaxSrcCount = lNum;
		} 
		else 
		{
			ptr->pDest = pMem;
			ptr->lMaxDestCount = lNum;
		}
		pMem[0] = '\0';	//ensure null terminated
	}
	
	//assign ptrs in sh structure.
	if (iSrcDest == SH_SRC_FILE)
	{
		ptr->pFrom = pMem;
		ptr->lpFileOp.pFrom = ptr->pFrom;

	} 
	else 
	{
		ptr->pTo = pMem;
		ptr->lpFileOp.pTo = ptr->pTo;
	}
	
}


/*
 * initialize a shell file operation
 */ 
void ShellFileOpInit(HWND hWnd, shell_op_t *ptr)
{
	ZeroMemory(ptr,sizeof(shell_op_t));
	ptr->lpFileOp.hwnd = hWnd;

}


/*
 * destroy a shell file operation
 */
void ShellFileOpDestroy(shell_op_t *ptr)
{
	//free all malloc'd blocks
    if (ptr->pTo) 
		GlobalFreePtr(ptr->pTo);
    if (ptr->pFrom)
		GlobalFreePtr(ptr->pFrom);
    if (ptr->pTitle)
		GlobalFreePtr(ptr->pTitle);
}

//
// long DeleteFiles ( )
//
// Function to perform delete operation
//
long ShellFileOpDeleteFiles(shell_op_t *ptr)
{
	ptr->lpFileOp.fAnyOperationsAborted = FALSE; 
	ptr->lpFileOp.wFunc = FO_DELETE;
	ptr->lpFileOp.fFlags =  FOF_ALLOWUNDO;
	return SHFileOperation(&ptr->lpFileOp);
}

//
// long MoveFiles ( )
//
// Function to perform move operation
//
long ShellFileOpMoveFiles(shell_op_t *ptr)
{
	ptr->lpFileOp.fAnyOperationsAborted = FALSE; 
	ptr->lpFileOp.wFunc = FO_MOVE;
	return SHFileOperation(&ptr->lpFileOp);
}

//
// long CopyFiles ( )
//
// Function to perform Copy operation
//
long ShellFileOpCopyFiles(shell_op_t *ptr)
{
	ptr->lpFileOp.fAnyOperationsAborted = FALSE; 
	ptr->lpFileOp.wFunc = FO_COPY;
	return SHFileOperation(&ptr->lpFileOp);
}

//
// long RenameFiles ( )
//
// Function to perform rename operation
//
long ShellFileOpRenameFiles(shell_op_t *ptr)
{
	ptr->lpFileOp.fAnyOperationsAborted = FALSE; 
	ptr->lpFileOp.wFunc = FO_RENAME;
	return SHFileOperation(&ptr->lpFileOp);
}

//
// void ClearFiles ( const int iSrcDest )
//
// Clears a list of files
//
void ShellFileOpClearFiles(int iSrcDest, shell_op_t *ptr)
{
	if (iSrcDest == SH_SRC_FILE)
	{
		if (ptr->pFrom) 
		{
			GlobalFreePtr(ptr->pFrom);
			ptr->pFrom = NULL;
			ptr->lpFileOp.pFrom = ptr->pFrom;
			ptr->lMaxSrcCount = 0;
			ptr->lCurSrcCount = 0;
		}	
	}
	else
	{
		if (ptr->pTo) 
		{
			GlobalFreePtr(ptr->pTo);
			ptr->pTo = NULL;
			ptr->lpFileOp.pTo = ptr->pTo;
			ptr->lMaxDestCount = 0;
			ptr->lCurDestCount = 0;
		}
	}
}

//
// void AddFile ( const int iSrcDest, char *sFile )
//
// Adds another filename to the end of the current string
//
void ShellFileOpAddFile(int iSrcDest, const char *sFile, shell_op_t *ptr)
{
	int iLength = lstrlen(sFile) + 1;	//+1 for the null

	if (iSrcDest == SH_SRC_FILE)
	{
		//check enough allocated space...
		if ((ptr->lCurSrcCount + iLength + 1)> ptr->lMaxSrcCount)	//+1 for the double null termination...
		{
			//have to get more mem.
			GrabMem(iSrcDest, (ptr->lCurSrcCount + iLength + 1), ptr);
		}
		//now theres enough memory!  yay.
		//now copy the filename in
		lstrcpy(ptr->pSrc, sFile);

		//go to end of this files null term.
		ptr->pSrc += iLength;
		ptr->lCurSrcCount += iLength;

		//always keep it double null terminated, but don't
		// increment past it incase we want to add more files
		ptr->pSrc[0] = 0;
	}
	else
	{
		//check enough allocated space...
		if ((ptr->lCurDestCount + iLength + 1)> ptr->lMaxDestCount)	//+1 for the double null termination...
		{
			//have to get more mem.
			GrabMem(iSrcDest, (ptr->lCurDestCount + iLength + 1),ptr);
		}
		//now theres enough memory!  yay.
		//now copy the filename in
		lstrcpy(ptr->pDest, sFile);

		//go to end of this files null term.
		ptr->pDest += iLength;
		ptr->lCurDestCount += iLength;
		//always keep it double null terminated, but don't
		// increment past it incase we want to add more files
		ptr->pDest[0] = 0;
	}
}

//
// SetMaxCount( const int iSrcDest, long lMax )
//
// Function to pre-allocate string memory to prevent 
//  lots of re-allocations
//
void ShellFileOpSetMaxCount(int iSrcDest, long lMax, shell_op_t *ptr)
{
	if (iSrcDest == SH_SRC_FILE)
	{
		ptr->lMaxSrcCount = lMax + 1;	//+1 for double null term.
		GrabMem(iSrcDest, ptr->lMaxSrcCount,ptr);
	} else {
		ptr->lMaxDestCount = lMax + 1;	//+1 for double null term.
		GrabMem(iSrcDest, ptr->lMaxDestCount,ptr);
	}
}

//
// SetTitle ( CString sTitle )
//
// Function to set the dialog title from a string
//
void ShellFileOpSetTitle(const char *sTitle, shell_op_t *ptr)
{
	int iLength;
	char * pBuf;

	//free mem of current title
	if (ptr->pTitle)
	{
		GlobalFreePtr(ptr->pTitle);
		ptr->pTitle = NULL;
		ptr->lpFileOp.lpszProgressTitle = NULL;
	}

	iLength = lstrlen(sTitle)+1;

	if (iLength > 1)
	{
		//grab more mem
		ptr->pTitle = (char *)GlobalAllocPtr(iLength);
		//copy the title
		pBuf = (char *)sTitle;
		lstrcpy(ptr->pTitle, pBuf);
		//now point the struct to the title
		ptr->lpFileOp.lpszProgressTitle = ptr->pTitle;
	}
}
