/*
 * $Id: utils.c,v 1.1 2003/11/14 10:03:37 alexanto Exp $
 *
 * $Doctor Gauge (C) 2000-2003 A. Antonini, All rights reserved $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Desc: Sundry Utilities $
 *
 * $Log: utils.c,v $
 * Revision 1.1  2003/11/14 10:03:37  alexanto
 * Initial revision
 *
 */
#include <windows.h>
#include "utils.h"

#define dwAllocFlags (GMEM_MOVEABLE | GMEM_ZEROINIT)
 
/*
 * trim white chars
 */
char *strrtrim(char *st)
{
	int i =strlen(st);
    while(st[i]<=' ') 
	{
		st[i]='\0';
        i--;
    }
	return st;
}


/*
 * copy a pointer from another
 */
void *GlobalDupePtr(void *src)
{
	void *ptr = NULL;
	HGLOBAL hSrc = GlobalHandle(src);
	DWORD dwSize = GlobalSize(hSrc);
	HANDLE hGlob =  GlobalAlloc(dwAllocFlags,dwSize);
	if (hGlob)
	{
		ptr = GlobalLock(hGlob);
		CopyMemory(ptr,src,dwSize);
	}
	return ptr;
}
 
 
/*
 * allocate a pointer
 */
void *GlobalAllocPtr(DWORD dwSize)
{
	HGLOBAL hGlob = GlobalAlloc(dwAllocFlags,dwSize);
    return (0==hGlob) ? NULL : GlobalLock(hGlob);

}

/*
 * reallocate a pointer
 */
void *GlobalReAllocPtr(void *buf, DWORD dwSize)
{

	HGLOBAL hGlob = GlobalHandle(buf);
	HGLOBAL hRealloc;
	GlobalUnlock(hGlob);
	hRealloc = GlobalReAlloc(hGlob,dwSize,dwAllocFlags);
	return (hRealloc==0) ? NULL : GlobalLock(hRealloc);
}

/*
 * free a pointer
 */
HGLOBAL GlobalFreePtr(void *buf)
{

	HGLOBAL hFree;
	HGLOBAL hGlob = GlobalHandle(buf);
	GlobalUnlock(hGlob);
	hFree = GlobalFree(hGlob);
	return hFree;
}


/*
 * A quick little routine that will center a window on the screen.
 * Handy for dialog boxes
 */
BOOL CenterWindow(HWND hwindow)
{
	RECT  rect;
    int w, h;
    int wScreen, hScreen, xNew, yNew;
    HDC dc;
    GetWindowRect(hwindow, &rect);
    w = rect.right - rect.left;
    h = rect.bottom - rect.top;

    dc = GetDC(hwindow);
	wScreen = GetDeviceCaps(dc, HORZRES);
	hScreen = GetDeviceCaps(dc, VERTRES);
	ReleaseDC(hwindow, dc);
    xNew = (wScreen >> 1) - (w >> 1);
    yNew = (hScreen >> 1) - (h >> 1);

    return SetWindowPos(hwindow, 0, xNew, yNew, 0, 0,
        SWP_NOSIZE | SWP_NOZORDER);
}


static const char *sFileInfo = "\\StringFileInfo\\040904B0\\";

void GetFileVersion(ver_info_t *ver_info)
{
	char exe_str[2048];
	char sFile[80];
	LPVOID lpBuffer;
	LPVOID lpData;
	DWORD dwSize, dwHandle = 0;
	UINT uiDataSize = 80;
	GetModuleFileName(NULL,exe_str,sizeof(exe_str));

	dwSize = GetFileVersionInfoSize(exe_str, &dwHandle);
	lpBuffer = GlobalAllocPtr(dwSize);

	// Get the versioninformation block
	GetFileVersionInfo(exe_str, 0, dwSize, lpBuffer);

	ZeroMemory(ver_info,sizeof(ver_info_t));
	
	lstrcpy(sFile,sFileInfo);
	lstrcat(sFile,"ProductName");

	VerQueryValue(lpBuffer, sFile, &lpData, &uiDataSize);
	lstrcpyn(ver_info->szProductName,(char *)lpData,
		min(sizeof(ver_info->szProductName),uiDataSize));

	lstrcpy(sFile,sFileInfo);
	lstrcat(sFile,"FileVersion");
	VerQueryValue(lpBuffer, sFile, &lpData, &uiDataSize);
	lstrcpyn(ver_info->szFileVersion,(char *)lpData,
		min(sizeof(ver_info->szFileVersion),uiDataSize));

	lstrcpy(sFile,sFileInfo);
	lstrcat(sFile,"LegalCopyRight");
	VerQueryValue(lpBuffer, sFile, &lpData, &uiDataSize);
	lstrcpyn(ver_info->szLegalCopyright,(char *)lpData,
		min(sizeof(ver_info->szLegalCopyright),uiDataSize));

	GlobalFree(lpBuffer);
}

char *VerifySlash(char *str)
{
	int i;
	if (str)
	{
		i = lstrlen(str);
		if (str[i-1]!='\\')
		{
			str[i]='\\';
			str[i+1]='\0';
		}
	}
	return str;
}

char *DeleteSlash(char *str)
{
	int i;
	if (str)
	{
		i = lstrlen(str);
		if (str[i-1]=='\\')
		{
			str[i-1]='\0';
		}
	}
	return str;
}

/*
 * insert a string onto another
 */
char *strins(const char *instr, char *str, int st_pos)
{

	int i, leninstr = lstrlen(instr);  // get length of string to insert

    // shift affected portion of str text to the right
    i = strlen(str);
    while (i>=st_pos)
	{
        *(str+leninstr+i) = *(str+i);
         i--;
    }

    // insert instr text
    for (i=0; i<leninstr; i++)
        *(str+st_pos+i) = *(instr+i);

	return str;
}

/*
 * get application path
 */
char *GetAppPath(char *str)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char name[_MAX_FNAME];
	char ext[_MAX_EXT];
	char buff[_MAX_PATH];
	GetModuleFileName(NULL,buff,sizeof(buff));
	_splitpath(buff,drive,dir,name,ext);
	lstrcpy(str,drive);
	lstrcat(str,dir);
	VerifySlash(str);
	return str;
}

char *ChangeFileExt(char *str, const char *new_ext)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char name[_MAX_FNAME];
	char ext[_MAX_EXT];
	char buff[_MAX_PATH];
	GetModuleFileName(NULL,buff,sizeof(buff));
	_splitpath(buff,drive,dir,name,ext);
	_makepath(str,drive,dir,name,new_ext);
	return str;
}

char *ExtractFilePath(char *str)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char name[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath(str,drive,dir,name,ext);
	_makepath(str,drive,dir,NULL,NULL);
	return str;
}

BOOL DirectoryExists(char *dir)
{
	DWORD dwCode = GetFileAttributes(dir);
	return (dwCode!=0xFFFFFFFF) && ((dwCode & FILE_ATTRIBUTE_DIRECTORY)==
		FILE_ATTRIBUTE_DIRECTORY);

}

BOOL ForceDirectories( char *Dir)
{
	char path[_MAX_PATH];
	char buff[_MAX_PATH];

	lstrcpy(path,Dir);
	if (*path)
	{
		DeleteSlash(path);
		lstrcpy(buff,path);
		ExtractFilePath(buff);
		if (lstrlen(path)<3 || DirectoryExists(path)
			|| 0 == lstrcmp(path,buff))
			return TRUE;
		return  ForceDirectories(buff) && CreateDirectory(path,NULL);
	}
	return FALSE;
}


/*
function ForceDirectories(Dir: string): Boolean;
begin
  Result := True;
  if Length(Dir) = 0 then
    raise Exception.CreateRes(@SCannotCreateDir);
  Dir := ExcludeTrailingPathDelimiter(Dir);
{$IFDEF MSWINDOWS}
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
{$ENDIF}
{$IFDEF LINUX}
  if (Length(Dir) = 0) or DirectoryExists(Dir) then Exit;
{$ENDIF}
  Result := ForceDirectories(ExtractFilePath(Dir)) and CreateDir(Dir);
end;
*/

/*
 * identifies the thousand separator                      
 */
char GetThousandSeparator(void)
{
	int i;
	char s[3];
    i = GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_STHOUSAND,s,2);
    if (i!=0) return *s;
    else return '.';
}


/*
 * insert thousand separator into a string
 */
char *str_insert_commas(char *str, int number)
{
	char csep = GetThousandSeparator();
	char s[2];
	int i;
	s[0]=csep;
	s[1]=0x0;
	i=wsprintf(str,"%d",number);
	
    while (i > 3)
	{
        i-=3;
		strins(s,str,i);
	}
	return str;
}

int GetIniInt( const char *Section, const char *Ident, int nDef)
{
	char ini_file[1024];
	int i;
	ChangeFileExt(ini_file,".ini");
	i = GetPrivateProfileInt(Section,Ident,nDef,ini_file);
	return i;
}

void SetIniInt(const char *Section, const char *Ident, int nVal)
{
	TCHAR ini_file[1024];
	TCHAR buff[64];
	ChangeFileExt(ini_file,".ini");
	wsprintf(buff,"%d",nVal);
	WritePrivateProfileString(Section,Ident,buff,ini_file);
}